/* --------------------------------------------*\
| ui_text.c (version 2.0) - (C) SichemSoft 1998 |
| Roghorst 160, 6708 KS Wageningen, Netherlands |
| module  for language independent applications |
| author: Anneke Sicherer-Roetman, date: 980522 |
\* --------------------------------------------*/

/* $Id$ */
/* $Log$ */

#define UIT_ENCRYPT 31

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define fileseek(pos)             (fp?!fseek(fp,pos,SEEK_SET):0)
#define filetell()                (fp?ftell(fp):-1)
#define fileread(buffer,num)      (fp?fread(buffer,num,1,fp)==1:0)

char **ui_text;                       /* pointers into memory text buffer */
unsigned short _ui_memcount,_ui_filcount; /* undocumented global variables */
static char *ui_textbuffer;           /* memory text buffer */
static unsigned long *ui_file;        /* pointers into file */
static char *ui_filebuffer;           /* file text buffer */
static char **ui_fileptr;             /* pointers into array from file */
static unsigned short itemchecksum=0; /* for checksumming file items */
static FILE *fp=NULL;                 /* file pointer */

/* converts string to all lower case (common ANSI extension) */
#pragma warn -sig /* Conversion may lose significant digits */
static char *strlwr(char *s)
{
   char *p;
   for (p=s; *s; s++) if (isupper(*s)) *s=tolower(*s);
   return p;
} /* strlwr */
#pragma warn .sig

/* decrypts text and computes checksum */
static unsigned short decrypt(char *buffer,unsigned long size)
{
   register unsigned long offset;
   unsigned short checksum;
   char *q;

   for (offset=0,checksum=0,q=buffer; offset<size; offset++,q++) {
      *q^=UIT_ENCRYPT; checksum+=(unsigned char)(*q);
   }
   return checksum;
} /* decrypt */

/* loads text from file into ui_filebuffer */
static char *loadtext(unsigned long pos,unsigned size)
{
   static unsigned long oldpos=(unsigned long)(-1L);

   if (pos==oldpos) return ui_filebuffer;
   if (!fileseek(pos)) return NULL;
   if (ui_filebuffer) free(ui_filebuffer);
   ui_filebuffer=(char *)malloc(size*sizeof(char));
   if (!ui_filebuffer) return NULL;
   if (!fileread(ui_filebuffer,size*sizeof(char))) return NULL;
   itemchecksum=decrypt(ui_filebuffer,size);
   oldpos=pos; /* next time only load if necessary */
   return ui_filebuffer;
} /* loadtext */

/* loads text (no array) from file into ui_filebuffer */
/* note: the programmer should not call this routine directly */
#pragma warn -sig /* Conversion may lose significant digits */
char *ui_filetext(unsigned pos)
{
   return loadtext(ui_file[pos],ui_file[pos+1]-ui_file[pos]);
} /* ui_filetext */

/* loads text array from file into ui_filebuffer */
/* note: the programmer should not call this routine directly */
char **ui_filearray(unsigned pos)
{
   unsigned size; register unsigned i;

   if (!fileseek(ui_file[pos])) return NULL;
   if (!fileread(&size,sizeof(size))) return NULL;
   if (!loadtext(filetell(),ui_file[pos+size]-ui_file[pos]-sizeof(size)))
      return NULL;
   if (ui_fileptr) free(ui_fileptr);
   ui_fileptr=(char **)malloc(size*sizeof(char*));
   if (!ui_fileptr) return NULL;
   ui_fileptr[0]=ui_filebuffer;
   for (i=1; i<size; i++)
      ui_fileptr[i]=ui_filebuffer+ui_file[pos+i]-ui_file[pos]-sizeof(size);
   return ui_fileptr;
} /* ui_filearray */

/* opens file, loads text and offsets into memory */
/* call this routine at the very beginning of the application */
#pragma warn -csu /* Comparing signed and unsigned values */
int ui_loadtext(char *fname,char *vers)
/* fname is name of desired .etf file without extension
   vers is optional version number, may also be "" */
{
   char *q;
   unsigned len,i;
   unsigned short checksum,checksum1=0,checksum2=0;
   char buf[256],file[81];
   unsigned long offset,size;

   if (ui_text || ui_file) return 0;

   /* open encrypted text file */
   strcpy(file,fname);
   if (!strchr(file,'.')) strcat(file,".etf");
   if ((fp=fopen(file,"rb+"))==NULL) return 0;

   /* read file header */
   if (!vers) vers="";
   q=strrchr(file,'\\'); if (q) q++; else q=file;
   len=strlen(q)+strlen(vers);
   if (!fileread(buf,(len+1)*sizeof(char))) return 0;
   strlwr(buf); strlwr(q); strlwr(vers);
   if (strncmp(buf,q,strlen(q))) return 0;
   if (strncmp(buf+strlen(q),vers,strlen(vers))) return 0;

   /* read number of items */
   if (!fileread(&checksum,sizeof(checksum))) return 0;
   if (!fileread(&_ui_memcount,sizeof(_ui_memcount))) return 0;
   if (!fileread(&_ui_filcount,sizeof(_ui_filcount))) return 0;
   if (!fileread(&size,sizeof(size))) return 0;
   if (_ui_memcount) {
      /* reserve space */
      ui_text=(char **)malloc(_ui_memcount*sizeof(char *));
      if (!ui_text) return 0;
      ui_textbuffer=(char *)malloc(size*sizeof(char));
      if (!ui_textbuffer) return 0;

      /* read offsets and compute pointers */
      for (i=0; i<_ui_memcount; i++) {
         if (!fileread(&offset,sizeof(offset))) return 0;
         ui_text[i]=ui_textbuffer+offset;
      }
   }

   if (_ui_filcount) {
      /* reserve space */
      ui_file=(unsigned long *)malloc((_ui_filcount+1)*sizeof(unsigned long));
      if (!ui_file) return 0;

      /* read offsets of file items */
      for (i=0; i<_ui_filcount+1; i++) {
         if (!fileread(&offset,sizeof(offset))) return 0;
         ui_file[i]=offset;
      }
   }

   if (_ui_memcount) {
      /* read text lines and decrypt */
      if (!fileread(ui_textbuffer,size*sizeof(char))) return 0;
      checksum1=decrypt(ui_textbuffer,size);
   }

   if (_ui_filcount) {
      /* read file items and decrypt for checksumming */
      for (i=0; i<_ui_filcount; i++) {
         q=ui_filetext(i); if (!q) return 0;
         checksum2+=itemchecksum;
      }
   }

   /* close file if no file items present */
   if (!_ui_filcount) if (fp) { fclose(fp); fp=NULL; }

   /* succesful load if checksum is ok */
   return (checksum==checksum1+checksum2);
} /* ui_loadtext */
#pragma warn .csu
#pragma warn .sig

/* frees all space and closes file if necessary */
/* call this routine at the very end of the application */
void ui_unloadtext(void)
{
   if (ui_text) { free(ui_text); ui_text=NULL; }
   if (ui_textbuffer) { free(ui_textbuffer);  ui_textbuffer=NULL; }
   if (ui_file) { free(ui_file); ui_file=NULL; }
   if (ui_filebuffer) { free(ui_filebuffer); ui_filebuffer=NULL; }
   if (ui_fileptr) { free(ui_fileptr); ui_fileptr=NULL; }
   if (fp) { fclose(fp); fp=NULL; }
} /* ui_unloadtext */


